<?php

namespace FivePm;

class Api
{
    const API_VERSION = 'v2';
    private $error;
    /**
     * 5pm account host
     * @var string $host
     */
    private $host;
    /**
     * debug mode flag
     * @var bool
     */
    private $debug;
    /**
     * api modules SoapClient
     * @var \SoapClient[] $clients
     */
    private $clients;
    /**
     * @var string $sessionId
     */
    private $sessionId;

    /**
     * @param $host
     * @param bool $debug
     */
    public function __construct($host, $debug = false)
    {
        if( $curl = curl_init() ) {
            curl_setopt($curl, CURLOPT_URL, 'https://' . $host);
            curl_setopt($curl,CURLOPT_RETURNTRANSFER,true);
            curl_setopt($curl,CURLOPT_NOBODY,true);
            curl_setopt($curl,CURLOPT_HEADER,true);
            $out = curl_exec($curl);
            $out = explode("\r\n", $out);
            foreach($out as $params) {
                if(strpos($params, 'Location') === 0){
                    $location = explode(':', $params);
                    $url = trim($location[1].':'.$location[2]);
                }
                if(strpos($params, '200 OK')){
                    $ok = true;
                }
            }
            curl_close($curl);
        }

        if(strpos($url, 'login.php') > 0 || (!$url && $ok)){
            $this->host       = $host;
            $this->debug      = (boolean)$debug;
            $this->clients    = array();
            $this->sessionId  = null;
            $this->error      = null;
        } else { 	    
            $this->error = 'Domain not found '; 
        }
    }

    /**
     * @param $clientName
     * @return \SoapClient
     */
    public function __get($clientName)
    {
        if (trim($this->host)=='')  { 
            $this->error = 'Domain not found'; 
            return false;
        }
        $clientName = strtolower($clientName);

        if (!isset($this->clients[$clientName])) {
            $wsdlUrl = 'https://' . $this->host
                . '/api/' . self::API_VERSION
                . '/wsdl/' . $clientName . '.wsdl?8';
            $this->clients[$clientName] = new \SoapClient($wsdlUrl, array('trace' => $this->debug, 'exceptions' => 1));
            if ($this->isAuthorized()) {
                $this->clients[$clientName]->__setsoapHeaders(
                    array(new \SoapHeader('http://' . $this->host . '/api/'.self::API_VERSION.'/wsdl/schema', 'sessionId', $this->sessionId))
                );
            }

        }

        return $this->clients[$clientName];
    }

    public function __set($name, $value)
    {
        // pass
    }
    public function getError() {
        return $this->error;
    }
    /**
     * @return bool
     */
    public function isAuthorized()
    {
        return '' != $this->sessionId;
    }

    /**
     * @param string $username
     * @param string $password
     * @throws \SoapFault
     * @return bool
     */
    public function signIn($username, $password)
    {
        /** @noinspection PhpUndefinedFieldInspection */
        $result          = $this->authentication->signIn($username, $password);
        $this->sessionId = is_object($result) ? $result->sessionId : $result;
        $this->resetSessionHeader();
        return $this->isAuthorized();
    }

	public function setSessionId($sessId)
    {
        $this->sessionId = $sessId;
    }
	
    /**
     * @return void
     */
    public function signOut()
    {
        if ($this->sessionId) {
            /** @noinspection PhpUndefinedFieldInspection */
            $this->authentication->signOut();
            $this->sessionId = null;
            $this->resetSessionHeader();
        }
    }

    /**
     * @return string
     */
    public function getSessionId()
    {
        return $this->sessionId;
    }

    /**
     * @return void
     */
    private function resetSessionHeader()
    {
        foreach ($this->clients as $client) {
            if ($this->sessionId) {
                $client->__setSoapHeaders(
                    new \SoapHeader('http://' . $this->host . '/api/'.self::API_VERSION.'/wsdl/schema', 'sessionId', $this->sessionId)
                );
            } else {
                $client->__setSoapHeaders(null);
            }
        }
    }
}
